(use-translation _ "alterator-network")

;general all possible ipv4 netmasks
(define (forth n) (bit-extract n 0 8))
(define (third n) (bit-extract n 8 16))
(define (second n) (bit-extract n 16 24))
(define (first n) (bit-extract n 24 32))
(define (ones n) (- (expt 2 n) 1))
(define (gen-mask n) (- (ones 32) (ones (- 32 n))))
(define (seq)
  (let loop ((start 32)
	     (stop 1)
	     (result '()))
    (if (>= start stop)
      (loop (- start 1) stop (append result (list start)))
      result)))

(define *netmasks* (map (lambda (x)
			 (let ((m (gen-mask x)))
			   (format #f "/~A    (~A.~A.~A.~A)"
				   x
				   (first m)
				   (second m)
				   (third m)
				   (forth m))))
		       (seq)))

(on-apply (ask-and-apply))

(define (on-load-error reason)
  '())

(define (get-ifacelist)
  (woo-cache-ignore)
  (woo-read-names "/simple_etcnet/ifaces"
		  (woo-list "/simple_etcnet/ifaces")))

(define (get-iface-info name)
  (woo-first-command (woo-read (string-append "/simple_etcnet/ifaces/" name))))

(define *ifaces* (woo-catch get-ifacelist on-load-error))
(define *first-item* (and (not (null? *ifaces*))
			  (woo-catch
			    (lambda () (get-iface-info (car *ifaces*)))
			    (lambda (reason) #f))))

(define (make-ifacelist)
  (id 'ifaces (listbox
		(relative-width 20 page)
		(relative-height 50 page)
		(items *ifaces*)
		current 0
		(on-select (update-ifaceconfig)))))

(define (default-value x cmd) (woo-get-option cmd x))

(define (get-default-ip cmd)
    (let ((ip (default-value 'ipv4address cmd)))
      (substring ip 0 (or (string-index ip #\/) (string-length ip)))))

(define (get-default-mask cmd)
  (if (null? ifaces)
    24; //24 netmask by default
    (let* ((ip (default-value 'ipv4address cmd))
	   (pos (string-index ip #\/))
	   (len (string-length ip)))
      (if (not pos)
	24 ;/24 netmask by default
	(string->number (substring ip (+ 1 pos) len))))))

(define (get-has-dhcp cmd) (string=? (default-value 'dhcp cmd) "yes"))

(define (enabled-settings? cmd) (not (or (get-has-dhcp cmd)
					 (null? *ifaces*))))

(define (ask-and-apply)
  (if (and (apply-changes enabled)
	   (run-dialog '("/net/apply_question")))
    (apply-ip)
    #t))

(define (update-ifaceconfig)
  (ask-and-apply)
  (update-information)
  (clear-dirty))

(define (update-information)
  (call-with-current-continuation
    (lambda (exit)
      (let ((new-result (woo-catch
			  (lambda () (get-iface-info (ifaces text)))
			  (lambda (reason)
			    (description text (string-append
						(_ "<b>Problem reading interface info:</b> ")
						reason))
			    (ip enabled #f)
			    (netmasks enabled #f)
			    (dhcp enabled #f)
			    (exit)))))
	
	(description text (default-value 'description new-result))
	(ip text (get-default-ip new-result))
	(netmasks current (- 32 (get-default-mask new-result)))
	(dhcp checked (get-has-dhcp new-result))
	(ip enabled (enabled-settings? new-result))
	(netmasks enabled (enabled-settings? new-result))))))

(define (on-apply-error reason)
  (if (string=? reason "wrong address")
    (run-dialog `("/net/error" reason "wrong address"))
    (run-dialog '("/error")))
  #f)

(define *selected-iface* #f)
(define (cmd-selected-iface)
  (string-append "/simple_etcnet/ifaces/" *selected-iface*))

(define (apply-ip)
  (woo-catch
    (lambda()
      (with-splash-message
	(lambda()
	  (splash-message (_ "restarting network ... "))
	  (if (dhcp checked)
	    (woo-write (cmd-selected-iface) 'dhcp "yes" 'use_hotplug (global 'use_hotplug))
	    (woo-write (cmd-selected-iface)
		       'dhcp "no"
		       'use_hotplug (global 'use_hotplug)
		       'ip (ip text)
		       'mask (- 32 (netmasks current))))))
      (clear-dirty)
      #t)
    on-apply-error))

(define (clear-dirty)
  (apply-changes enabled #f))

(define (mark-as-dirty)
  (set! *selected-iface* (ifaces text))
  (apply-changes enabled #t))

(define (update-enabled)
  (define enabled? (not (dhcp checked)))
  (ip enabled enabled?)
  (netmasks enabled enabled?)
  (mark-as-dirty))

(define (make-ifaceconfig)
  (vbox
    (id 'description (textbox (default-value 'description *first-item*)
			      (relative-width 70 page)
			      (relative-height 10 page)
			      readonly "yes"))
    (id 'dhcp (checkbox (_ "Configure automatically (DHCP)")
			checked (get-has-dhcp *first-item*)
			enabled (not (null? *ifaces*))
			(on-click (update-enabled))))
    (hbox
      (vbox (label (_ "IP Address:") height 3 enabled (enabled-settings? *first-item*))
	    (label (_ "Network Mask:") height 3 enabled (enabled-settings? *first-item*)))
      (vbox (id 'ip (edit (get-default-ip *first-item*)
			  (relative-width 50 page)
			  height 3
			  enabled (enabled-settings? *first-item*)
			  (on-change (mark-as-dirty))))
	    (id 'netmasks (combobox (items *netmasks*)
				    current (- 32 (get-default-mask *first-item*))
				    (relative-width 50 page)
				    height 3
				    enabled (enabled-settings? *first-item*)
				    (on-change (mark-as-dirty))))))
    (id 'apply-changes (button (_ "Apply changes")
			       (relative-width 30 page)
			       enabled #f
			       (on-click (apply-ip))))))


(vbox sizepolicy 'fixed
  (label (_ "Available network devices:"))
  (hbox spacing 1
    (make-ifacelist)
    (make-ifaceconfig)))

